import React from "react";
import {AutoColumnContextProvider, ColumnHeading, ColumnKey, Columns} from "./Column";
import {SmartableProperties, useTable} from "./Smartable";
import {RowInstance, RowLoader} from "./Row";
import {useAsyncManager} from "./AsyncManager";

/**
 * Smartable instance component properties.
 */
export interface InstanceProperties<CK extends ColumnKey> extends SmartableProperties<CK>
{
	/**
	 * Table columns.
	 */
	columns: Columns<CK>;
}

/**
 * Main component for a Smartable table.
 */
export function Instance<CK extends ColumnKey>({columns}: InstanceProperties<CK>)
{
	return (
		<table>
			<thead>
			<ColumnsHeadings columns={columns} />
			</thead>
			<tbody>
			<TableBody />
			</tbody>
		</table>
	);
}

/**
 * Columns headings of a Smartable table.
 */
export function ColumnsHeadings<CK extends ColumnKey>({columns}: {columns: Columns<CK>})
{
	return (
		<tr className={"headings"}>
			{ // Showing title of each column.
				Object.keys(columns).map((key) => (
					<AutoColumnContextProvider key={key} columnKey={key}>
						<ColumnHeading />
					</AutoColumnContextProvider>
				))
			}
		</tr>
	);
}

export function TableBody<CK extends ColumnKey>()
{
	// Get data from table.
	const {data} = useTable();

	// Get current data state from the async table value.
	const {currentDataState} = useAsyncManager(data);

	return (
		Array.isArray(currentDataState?.rows) ? (
			currentDataState.rows.map((rowData, index) => (
				// Rendering each row from its definition.
				<RowInstance row={rowData} />
			))
		) : (
			<RowLoader />
		)
	);
}
